/*
Create a matrix of randomly generated numbers and letters (WORDAMOUNT * STRINGLENGTH)
Find the numbers belonging to each number system (MINNUMBERSYSTEM to MAXNUMBERSYSTEM).
*/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>

#define WORDLENGTH 4 //Use for parsing
#define STRINGLENGTH WORDLENGTH + 1 //Use for memory allocation
#define MINNUMBERSYSTEM 2
#define MAXNUMBERSYSTEM 16
#define WORDAMOUNT 256
#define MAXPRINTINONEROW 75 // symbols

void fillStringArray(char[][STRINGLENGTH]);
void createRandomString(char[]);
int checkNumberSystem(char, int);
int fillOneSystem(char[][STRINGLENGTH], char[][STRINGLENGTH], int);
void fillAllSystems(char[][STRINGLENGTH], char[][WORDAMOUNT][STRINGLENGTH], int[]);
void output(char[][WORDAMOUNT][STRINGLENGTH], int[]);
void printGeneratedData(char[][STRINGLENGTH]);

int main(void){
    srand(time(NULL));
    if(WORDAMOUNT < 2 || WORDLENGTH < 4 || MAXNUMBERSYSTEM < 2)
        return 1;
    char numbers[WORDAMOUNT][STRINGLENGTH];
    char systemsArr[MAXNUMBERSYSTEM-MINNUMBERSYSTEM+1][WORDAMOUNT][STRINGLENGTH];
    int lengths[MAXNUMBERSYSTEM-MINNUMBERSYSTEM+1];
    fillStringArray(numbers);
    printGeneratedData(numbers);
    fillAllSystems(numbers, systemsArr, lengths);
    output(systemsArr, lengths);
    return 0;
}

void fillStringArray(char mainArr[][STRINGLENGTH]){
    int i;
    for(i=0;i<WORDAMOUNT;i++){
        char temp[STRINGLENGTH];
        createRandomString(temp);
        strcpy(mainArr[i], temp);
    }
}

void createRandomString(char str[]){
    int i;
    for(i=0;i<WORDLENGTH;i++){
        str[i] = rand() % MAXNUMBERSYSTEM + '0';
        if(str[i] > '9'){
            str[i] += ('A'-'9');
        }
    }
    str[WORDLENGTH] = '\0';
}

int checkNumberSystem(char symbol, int system){
    if(system <= 10){
        if(symbol >= (system)+'0')
            return 0;
    }else{
        if(symbol >= 'A')
            if(symbol >= (system-10)+'A')
                return 0;
    }
    return 1;
}

int fillOneSystem(char str[][STRINGLENGTH], char numberSystemStr[][STRINGLENGTH], int system){
    int i, j, s=0;
    for(i=0;i<WORDAMOUNT;i++){
        for(j=0;j<WORDLENGTH;j++){
            if(!checkNumberSystem(str[i][j], system))
                break;
        }
        if(j==WORDLENGTH)
            strcpy(numberSystemStr[s++], str[i]);
    }
    return s;
}

void fillAllSystems(char str[][STRINGLENGTH], char numberSystemsStr[][WORDAMOUNT][STRINGLENGTH], int systemStrLengths[]){
    int i;
    for(i=0;i<MAXNUMBERSYSTEM-MINNUMBERSYSTEM+1;i++){
        systemStrLengths[i] = fillOneSystem(str, numberSystemsStr[i], i+MINNUMBERSYSTEM);
    }
}

void output(char result[][WORDAMOUNT][STRINGLENGTH], int lengths[]){
    int i, j;
    for(i=0;i<MAXNUMBERSYSTEM-MINNUMBERSYSTEM+1;i++){
        if(lengths[i] > 0){
            int inOneRow = 12;
            printf("%d system: ", i+MINNUMBERSYSTEM);
            for(j=0;j<lengths[i];j++){
                printf("%s ", result[i][j]);
                inOneRow += WORDLENGTH+1;
                if(inOneRow >= MAXPRINTINONEROW && j != lengths[i] - 1){
                    inOneRow = 0;
                    printf("\n");
                }
            }
            printf("\n\n");
        }
    }
}

void printGeneratedData(char data[][STRINGLENGTH]){
    int i, inOneRow = 0;;
    for(i=0;i<WORDAMOUNT;i++){
        printf("%s ", data[i]);
        inOneRow += WORDLENGTH+1;
        if(inOneRow >= MAXPRINTINONEROW){
            inOneRow = 0;
            printf("\n");
        }
    }
    printf("\n\n");
}

